﻿using System;
using System.IO;
using System.Linq;
using Microsoft.VisualStudio.TestTools.UnitTesting;
using EasierUnitTestingSampleCode;

namespace TestProject1
{
	/// <summary>
	/// Summary description for UnitTest1
	/// </summary>
	[TestClass]
	public class UnitTest1
	{
		/// <summary>
		///Gets or sets the test context which provides
		///information about and functionality for the current test run.
		///</summary>
		public TestContext TestContext { get; set; }

		#region Additional test attributes
		//
		// You can use the following additional attributes as you write your tests:
		//
		// Use ClassInitialize to run code before running the first test in the class
		// [ClassInitialize()]
		// public static void MyClassInitialize(TestContext testContext) { }
		//
		// Use ClassCleanup to run code after all tests in a class have run
		// [ClassCleanup()]
		// public static void MyClassCleanup() { }
		//
		// Use TestInitialize to run code before running each test 
		// [TestInitialize()]
		// public void MyTestInitialize() { }
		//
		// Use TestCleanup to run code after each test has run
		// [TestCleanup()]
		// public void MyTestCleanup() { }
		//
		#endregion

		[TestMethod]
		[ExpectedException(typeof(ArgumentNullException))]
		public void Capitalize_Throws_Exception_When_Argument_Is_Null()
		{
			var component = new SampleComponent();
			component.CapitalizeThis(null);
		}

		public void ConfigData_Roundtrips_Successfully()
		{
			var configData = new ConfigData
								{
			                 		CurrentOperation = "test",
			                 		MaxRetries = 55,
			                 		StorageDirectory = @"C:\temp"
								};
			var repository = new ConfigDataFileRepository("sample.xml");
			repository.Save(configData);

			var loaded = repository.Load();

			Assert.AreEqual(configData.CurrentOperation, loaded.CurrentOperation);
			Assert.AreEqual(configData.MaxRetries, loaded.MaxRetries);
			Assert.AreEqual(configData.StorageDirectory, loaded.StorageDirectory);
		}

		public void ConfigData_Roundtrips_Successfully2()
		{
			var configData = new ConfigData
			{
				CurrentOperation = "test",
				MaxRetries = 55,
				StorageDirectory = @"C:\temp"
			};
			var repository = new ConfigDataFileRepository("sample.xml");
			var ms = new MemoryStream();
			repository.SaveToStream(configData, ms);

			ms.Position = 0;
			var loaded = repository.LoadFromStream(ms);

			Assert.AreEqual(configData.CurrentOperation, loaded.CurrentOperation);
			Assert.AreEqual(configData.MaxRetries, loaded.MaxRetries);
			Assert.AreEqual(configData.StorageDirectory, loaded.StorageDirectory);
		}

		[TestMethod]
		public void Validate_GetEvens()
		{
			var sample = new SampleComponent();
			var evens = sample.GetEvens(12, 4);

			Assert.AreEqual(12, evens[0]);
			Assert.AreEqual(14, evens[1]);
			Assert.AreEqual(16, evens[2]);
			Assert.AreEqual(18, evens[3]);
		}

		[TestMethod]
		public void Validate_GetEvens2()
		{
			var sample = new SampleComponent();
			var evens = sample.GetEvens(12, 4);

			CollectionAssert.AreEqual(new[] {12, 14, 16, 18}, evens);
		}

		[TestMethod]
		public void Find_Five_Temps_Over_100()
		{
			var sample = new SampleComponent();

			var results = sample.ComputeSamples();
			var query = from s in results
						where s.Temperature > 100.0
						select s;
			Assert.AreEqual(5, query.Count());
		}

		[TestMethod]
		public void Find_Four_Highest_Temps()
		{
			var sample = new SampleComponent();

			var results = sample.ComputeSamples();
			var query = (from s in results
							orderby s.Temperature descending 
							select s.Temperature).Take(4);
			CollectionAssert.AreEqual(
				new [] { 106.7, 106.2, 105.2, 103.9 }, query.ToArray());
		}
	}
}
